#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

from collections import OrderedDict

from pyams_content.interfaces.container import IOrderedContainer
from pyams_sequence.interfaces import IInternalReference
from zope.annotation import IAttributeAnnotatable

from pyams_i18n.schema import I18nTextLineField
from pyams_sequence.schema import InternalReferenceField
from zope.container.constraints import containers, contains
from zope.interface import invariant, Interface, Invalid
from zope.schema import Bool, Choice, Datetime, Int, URI
from zope.schema.vocabulary import SimpleVocabulary, SimpleTerm


__docformat__ = 'restructuredtext'

from pyams_content import _


ALERT_CONTAINER_KEY = 'pyams_content.alerts'


ALERT_GRAVITY_NAMES = OrderedDict((
    ('alert', _("Alert")),
    ('alert_end', _("End of alert")),
    ('info', _("Information")),
    ('warning', _("Warning")),
    ('recommend', _("Recommendation"))
))

ALERT_GRAVITY_VOCABULARY = SimpleVocabulary([SimpleTerm(v, title=t) for v, t in ALERT_GRAVITY_NAMES.items()])


class IAlertItem(IAttributeAnnotatable, IInternalReference):
    """Alert item interface"""

    containers('.IAlertContainer')

    visible = Bool(title=_("Visible?"),
                   description=_("Is this alert visible in front-office?"),
                   required=True,
                   default=False)

    gravity = Choice(title=_("Alert gravity"),
                     description=_("Alert gravity will affect rendered alert style"),
                     required=True,
                     default='info',
                     vocabulary=ALERT_GRAVITY_VOCABULARY)

    message = I18nTextLineField(title=_("Message"),
                                description=_("Alert message"),
                                required=True)

    reference = InternalReferenceField(title=_("Internal reference"),
                                       description=_("Internal link target reference. You can search a reference using "
                                                     "'+' followed by internal number, of by entering text matching "
                                                     "content title."),
                                       required=False)

    external_url = URI(title=_("External URL"),
                       description=_("Alternate external URL"),
                       required=False)

    @invariant
    def check_url(self):
        if self.reference and self.external_url:
            raise Invalid(_("You can't set internal reference and external URI simultaneously!"))

    start_date = Datetime(title=_("Display start date"),
                          description=_("First date at which alert should be displayed"),
                          required=False)

    end_date = Datetime(title=_("Display end date"),
                        description=_("Last date at which alert should be displayed"),
                        required=False)

    maximum_interval = Int(title=_("Maximum interval"),
                           description=_("Maximum interval between alert displays on a given device, "
                                         "given in hours; set to 0 to always display the alert"),
                           required=True,
                           min=0,
                           default=48)


class IAlertContainer(IOrderedContainer):
    """Alert container interface"""

    contains(IAlertItem)

    def append(self, value, notify=True):
        """Append given value to container"""

    def get_visible_items(self):
        """Get list of visible items"""


class IAlertTarget(Interface):
    """Alert container target interface"""
